package cn.bunny.controller;

import cn.bunny.service.SysUserService;
import cn.bunny.service.ValidateCodeService;
import cn.bunny.spzx.model.dto.system.LoginDto;
import cn.bunny.spzx.model.entity.system.SysUser;
import cn.bunny.spzx.model.vo.common.Result;
import cn.bunny.spzx.model.vo.common.ResultCodeEnum;
import cn.bunny.spzx.model.vo.system.LoginVo;
import cn.bunny.spzx.model.vo.system.ValidateCodeVo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@Tag(name = "用户接口")
@RestController
@RequestMapping("/admin/system/index")
public class IndexController {
    @Autowired
    private SysUserService sysUserService;

    @Autowired
    private ValidateCodeService validateCodeService;

    // 用户登录
    @Operation(summary = "用户登录", description = "用户登录获取token")
    @PostMapping("login")
    public Result login(@RequestBody LoginDto loginDto) {
        LoginVo loginVo = sysUserService.login(loginDto);
        return Result.build(loginVo, ResultCodeEnum.LOGIN_SUCCESS);
    }

    // 生成验证码
    @Operation(summary = "获取验证码", description = "会获得两个值：codeKey + codeValue")
    @GetMapping(value = "/generateValidateCode")
    public Result<ValidateCodeVo> generateValidateCode() {
        ValidateCodeVo validateCodeVo = validateCodeService.generateValidateCode();
        return Result.build(validateCodeVo, ResultCodeEnum.SUCCESS);
    }

    // 获取用户登录信息
    @Operation(summary = "获取用户登录信息", description = "获取token")
    @GetMapping(value = "getUserInfo")
    // public Result getUserInfo(HttpServletRequest httpServletRequest) {
    //     String token = httpServletRequest.getHeader("token");
    public Result getUserInfo(@RequestHeader(name = "token") String token) {
        SysUser sysUser = sysUserService.getUserInfo(token);
        return Result.build(sysUser, ResultCodeEnum.SUCCESS);
    }

    // 用户退出
    @Operation(summary = "用户退出", description = "清除Redis中token")
    @GetMapping(".logout")
    public Result logout(@RequestHeader(name = "token") String token) {
        sysUserService.logout(token);
        return Result.build(null, ResultCodeEnum.SUCCESS);
    }
}
