package cn.bunny.interceptor;

import cn.bunny.AuthContextUtil;
import cn.bunny.spzx.model.entity.system.SysUser;
import cn.bunny.spzx.model.vo.common.Result;
import cn.bunny.spzx.model.vo.common.ResultCodeEnum;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.lang.Nullable;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.concurrent.TimeUnit;

@Component
public class LoginAuthInterceptor implements HandlerInterceptor {
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    /**
     * 响应208给前端
     *
     * @param response
     */
    public void respondNoLoginInfo(HttpServletResponse response) {
        Result<Object> result = Result.build(null, ResultCodeEnum.LOGIN_AUTH);
        PrintWriter writer = null;
        response.setCharacterEncoding("UTF-8");
        response.setContentType("text/html; charset=utf-8");
        try {
            writer = response.getWriter();
            writer.println(JSON.toJSONString(result));
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (writer != null) writer.close();
        }
    }


    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        /**
         * 1. 获取骑牛方式
         *  - 如果请求方式是option 预检请求，直接做放行
         * 2. 从请求中获取token
         * 3. 如果token为空，返回错误提示
         * 4. 如果token不为空，拿着token查询redis
         * 5. 如果redis查询不到数据，返回错误提示
         * 6. 如果redis查询用户信息，把用户信息放到ThreadLocal
         * 7. 更新redis更新时间
         * 放行
         */

        //  1. 获取骑牛方式
        String method = request.getMethod();
        if ("OPTIONS".equals(method)) {
            return true;
        }

        // 2. 从请求中获取token
        String token = request.getHeader("token");

        // 3. 如果token为空，返回错误提示
        if (StrUtil.isEmpty(token)) {
            respondNoLoginInfo((response));
            return false;
        }

        // 4. 如果token不为空，拿着token查询redis
        String userinfoString = redisTemplate.opsForValue().get("user:login" + token);

        // 如果redis查询不到数据返回错误提示
        if (StrUtil.isEmpty(userinfoString)) {
            respondNoLoginInfo(response);
            return false;
        }

        // 如果redis查询到用户信息，将信息放到ThreadLocal中
        SysUser sysUser = JSON.parseObject(userinfoString, SysUser.class);
        AuthContextUtil.set(sysUser);

        // 吧redis用户信息数据更新过期时间
        redisTemplate.expire("user:login" + token, 30, TimeUnit.MINUTES);

        return true;
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, @Nullable Exception ex) throws Exception {
        AuthContextUtil.remove();
    }
}
